<?php
/*
 * *****************************************************************************
 * Lehrbuchausleihen nach Signatur
 * *****************************************************************************
 * - Februar 2006 OM
 * - basierdend auf dem Perl Script vom 18.06.01 Beate
 * - Übername Intranet: 27.02.2015 /TZ
 *
 * @note:
 * - Variante von http://intranet.b.tu-harburg.de/mediawiki/index.php/Fachreferenten:_Lehrbuchsammlung
 *
 * a) Schon ein join nur zwischen volume_statistics und volumes mit Summen
 * nach volume_number dauert ohne Jahresangabe 35 Sekunden. Beste Workaround:
 * Query nur auf volume_statistics mit iln+volume_number+jahr (=index)
 *
 * b) volumes.cum_loans etc. scheint nur loan3-Ausleihen zu umfassen, daher
 * wertlos
 *
 * c) Group by author killt die Performance (bzw. geht nur mit "Kürzung") - theoretisch.
 * Praktisch: Subquery funzt ganz gut
 * http://manuals.sybase.com/onlinebooks/group-as/asg1250e/svrtsg/@Generic__BookTextView/15165;pt=15380
 *
 * @todo:
 * - sanitize request
 */
// Datenbankverbindung
require_once('../../inc/lbs-conf.php');


/*##############################################################################
 * Variabeln von Formular oder URL-Parametern via POST
 #############################################################################*/
$now   = date("d.m.Y H:i");
$signatur = (isset($_REQUEST["opt_location"])) ? $_REQUEST["opt_location"] : 'LBS:AG';
$title = "Ausleihen für Signatur $signatur";


/*##############################################################################
 * Filterung (Formular)
 #############################################################################*/
// Helper table; maybe create groups
function tbl_td_class($td_value, $prefix = '') {
  if ($td_value == 0) {
    $class = 'class="'.$prefix.'val0"';
  }
  else {
    $class = 'class="'.$prefix.'val1"';
  }

  return $class;
}

//Semi-automatisches zusammensetzen der Standort-Signaturen
function frm_get_locations() {
  global $conn, $iln;

  $query = "SELECT
            	s.description 'Standort',
            	s.storage_room_code 'Standort-Code',
            	upper(ltrim(str_replace(substring(c.shelf_mark_start, 1, CHARINDEX(' ', c.shelf_mark_start + ' ')), '0000000', ''))) 'Systematik',
            	upper(substring(c.shelf_mark_start, 1, CHARINDEX(' ', c.shelf_mark_start + ' '))) 'Mag-Real',
            	c.explanation 'Systematik: Beschreibung'
            FROM
            	concordance c,
            	storage_room s
            WHERE
            	s.iln = $iln
            	AND c.iln=s.iln
            	AND c.storage_room_code=s.storage_room_code
            ORDER BY
            	s.storage_room_code,
            	c.shelf_mark_start
            ";
  $result = sybase_query("$query", $conn) or die ("# ERROR: Query in Sybase failed $now \n"); // todo mail an admin

  // Get everything - need LS1 + LS2 for LBS
  $select = '';
  while ($row = sybase_fetch_array($result)) {
    $location = trim($row['Standort-Code']);
    @$loc[$location]['lbl_optgroup'] = utf8_encode(trim($row['Standort-Code']).' -- '.trim($row['Standort']));
    @$loc[$location]['option_txt'][]  .= utf8_encode(trim($row['Systematik']).' -- '.$row['Systematik: Beschreibung']);
    @$loc[$location]['option_val'][]  .= utf8_encode(trim($row['Systematik']));
  }

//echo "<pre>";
//print_r($loc);

  $select .= '<select name="opt_location" id="opt_location">';
  foreach($loc AS $location => $frm_part) {
    $select .= '<optgroup label="'.$frm_part['lbl_optgroup'].'">';
    if ($location != 'LBS' && $location != 'LS1' && $location != 'LS2') {
      foreach ($loc[$location]['option_txt'] as $id => $option_txt) {
        $value = $loc[$location]['option_val'][$id];
        $value = $value.':';
      	$select .= '<option value="'.$value.'">'.$option_txt.'</option>';
      }
    }
    elseif ($location == 'LS1' || $location == 'LS2') {
      foreach ($loc[$location]['option_txt'] as $id => $option_txt) {
        $value = $loc[$location]['option_val'][$id];
        $value = ($value == 'EI' || $value == 'ROTUNDE' || $value == 'KF') ? $value.':' : $value.':'.$value;
      	$select .= '<option value="'.$value.'">'.$option_txt.'</option>';
      }
    }
    elseif ($location == 'LBS') {
      $ls = array_merge($loc['LS1']['option_val'], $loc['LS2']['option_val']);
      sort($ls);
      foreach ($ls as $option_val) {
        $value = 'LBS:'.$option_val;
      	$select .= '<option value="'.$value.'">'.$value.'</option>';
      }
    }
    $select .= '</optgroup>';
  }

  return $select;
}


$form  = '<div class=\"no_print\">
          <form method="post" action="">';
$form .=    frm_get_locations();
$form .= '  <input type="submit" value="Abschicken" />
          </form>
        </div>';


/*##############################################################################
 * JavaScript: Helper
 #############################################################################*/
$overlay = '<!-- overlayed element -->
            <div class="apple_overlay" id="overlay">
              <!-- the external content is loaded inside this tag -->
            <div class="contentWrap"></div>
            </div>';


/*##############################################################################
 * Query building & processing
 #############################################################################*/
$table = '';
if ($signatur) {
  $query = "SELECT
            	c.ppn,
            	c.signature 'Signatur',
            	substring(c.shorttitle, 1, 35) 'Titel',
            --	c.author 'Autor',
            		(SELECT DISTINCT substring(self.author, 1, patindex('% %', self.author)+1) || '.' FROM ous_copy_cache self
              		WHERE self.iln = $iln AND self.ppn = c.ppn AND self.signature = c.signature AND self.loan_indication = c.loan_indication
              		GROUP BY self.ppn, self.author, self.signature, self.loan_indication
              		HAVING count(self.ppn) = count(self.author)) 'Autor', --weird, why more than one row sometimes (e.g. for CH)?
            	coalesce(substring(a.isbn, 1, patindex('%[^-0-9X]%', upper(a.isbn))), '') 'ISBN',
            --	a.year 'Erscheinungsjahr',
            	substring(a.year, patindex('%[0-9][0-9][0-9][0-9]%', a.year), 4) 'Von',
            	datepart(yy, getdate()) - cast(substring(a.year, patindex('%[0-9][0-9][0-9][0-9]%', a.year), 4) as int) 'Alter',
            	COUNT(*) 'Exemplare',
            	(SELECT distinct lbs.description FROM lbs_code_desc lbs WHERE lbs.iln = $iln AND lbs.code_id = 5 AND convert(int, lbs.code_value) = c.loan_indication) 'Indikator',
            	(coalesce(sum(s0.cum_loans), 0) +
            	 coalesce(sum(s1.cum_loans), 0) +
            	 coalesce(sum(s2.cum_loans), 0) +
            	 coalesce(sum(s3.cum_loans), 0) +
            	 coalesce(sum(s4.cum_loans), 0) +
            	 coalesce(sum(s5.cum_loans), 0)) 'Ausleihen 5 Jahre', --volle 5 jahre und dieses jahr
            	(convert(numeric(15,2), (coalesce(sum(s0.cum_loans), 0) +
            	 coalesce(sum(s1.cum_loans), 0) +
            	 coalesce(sum(s2.cum_loans), 0) +
            	 coalesce(sum(s3.cum_loans), 0) +
            	 coalesce(sum(s4.cum_loans), 0) +
            	 coalesce(sum(s5.cum_loans), 0))) / COUNT(*)) '5J-Umsatz/Ex',
            	coalesce(sum(s0.cum_loans), 0) 'Ausleihen lfd. Jahr',
            	coalesce(sum(s0.cum_reservations), 0) 'Vormerk. lfd. Jahr',
            	coalesce(sum(s1.cum_loans), 0) 'Ausleihen J-1',
            	coalesce(sum(s1.cum_reservations), 0) 'Vormerk. J-1',
            	coalesce(sum(s2.cum_loans), 0) 'Ausleihen J-2',
            	coalesce(sum(s2.cum_reservations), 0) 'Vormerk. J-2',
            	coalesce(sum(s3.cum_loans), 0) 'Ausleihen J-3',
            	coalesce(sum(s3.cum_reservations), 0) 'Vormerk. J-3',
            	coalesce(sum(s4.cum_loans), 0) 'Ausleihen J-4',
            	coalesce(sum(s4.cum_reservations), 0) 'Vormerk. J-4',
            	coalesce(sum(s5.cum_loans), 0) 'Ausleihen J-5',
            	coalesce(sum(s5.cum_reservations), 0) 'Vormerk. J-5'
            FROM
            	ous_copy_cache c
            	INNER JOIN volume v 				 ON (c.iln = v.iln AND c.epn = v.epn)
            	INNER JOIN acq_copy_cache a  ON (v.iln = a.iln AND v.epn = a.epn)
            	LEFT OUTER JOIN volume_statistics s0 ON (v.volume_number = s0.volume_number AND s0.year = datepart(yy, getdate()))
            	LEFT OUTER JOIN volume_statistics s1 ON (v.volume_number = s1.volume_number AND s1.year = datepart(yy, getdate())-1)
            	LEFT OUTER JOIN volume_statistics s2 ON (v.volume_number = s2.volume_number AND s2.year = datepart(yy, getdate())-2)
            	LEFT OUTER JOIN volume_statistics s3 ON (v.volume_number = s3.volume_number AND s3.year = datepart(yy, getdate())-3)
            	LEFT OUTER JOIN volume_statistics s4 ON (v.volume_number = s4.volume_number AND s4.year = datepart(yy, getdate())-4)
            	LEFT OUTER JOIN volume_statistics s5 ON (v.volume_number = s5.volume_number AND s5.year = datepart(yy, getdate())-5)
            WHERE
            	c.iln = $iln
            	AND upper(c.signature) LIKE '$signatur%'
            --	AND substring(upper(c.signature), 1, 4) = 'LBS:'
            -- 	AND substring(upper(c.signature), 5, 2) = 'TI'
            GROUP BY
            	c.ppn, c.signature, c.shorttitle, a.year, a.isbn, c.loan_indication
            ORDER BY
            	c.signature
            ";
//echo "<pre>$query</pre>";
  $result = sybase_query($query, $conn) or die ("# ERROR: Query in Sybase failed $now \n"); // todo mail an admin

  $table  = "<table id=\"myTable\" class=\"tablesorter display\">
              <thead>
                <tr>
                  <th colspan=\"3\">Bibliothek</th>
                  <th colspan=\"4\">Bibliographische Daten</th>
                  <th colspan=\"2\">Bedenken</th>
                  <th colspan=\"2\">Ab ".(date('Y')-5)."</th>
                  <th colspan=\"6\">Ausleihen</th>
                  <th colspan=\"2\">Vormerk.</th>
                </tr>
                <tr>
                <th>PPN</th>
                <th>Signatur</th>
                <th>Indikator</th>
                <th>Titel</th>
                <th>Autor</th>
                <th>ISBN</th>
                <th>Von</th>
                <th>Alter</th>
                <th>#Ex.</th>
                <th>Ausleihen</th>
                <th>Umsatz/Ex</th>
                <th>a".(date('Y'))."</th>
                <th>a".(date('Y')-1)."</th>
                <th>a".(date('Y')-2)."</th>
                <th>a".(date('Y')-3)."</th>
                <th>a".(date('Y')-4)."</th>
                <th>a".(date('Y')-5)."</th>
                <th>v".(date('Y'))."</th>
                <th>v".(date('Y')-1)."</th>
              </tr></thead>
              <tbody>";

  $sum_titles = $sum_volumes = $sum_avg = 0;
  while ($row = sybase_fetch_array($result)) {
    $avg = number_format ($row['5J-Umsatz/Ex'], 2, ",", ".");
    $clean_aut  = ($row['Autor'] == '') ? '<b>TZ zu doof</b>' : utf8_encode($row['Autor']);
    $clean_isbn = str_replace(',', '', $row['ISBN']); //patindex sux
    $clean_ppn  = (strlen($row['ppn']) == 7) ? '0'.$row['ppn'] : $row['ppn']; //nochmal ordentlich machen

    //$overlay_amazon = '<a href="http://www.amazon.de/s?field-keywords='.$row['ISBN'].'" class="fancyurl fancybox.ajax">A</a>';
    $overlay_buchkatalog = (strlen($clean_isbn) > 2) ? '<a href="http://www.buchkatalog.de/kod-bin/isuche.cgi?naviggif=Schnellsuche&aktion=start&fromQuicksearch=ja&lang=deutsch&uid=KNO-05032008-143504152-F06124&typqsrch=OPTIMIERT_KNOKV_ALLDB&dblang=&location=Profisuche&quicksearch='.$clean_isbn.'" class="fancyurl fancybox.iframe fi-info"> </a>' : '';
    $overlay_katalog = '<a href="https://katalog.b.tu-harburg.de/DB=1/CMD?ACT=SRCHA&IKT=1016&TRM=ppn+'.$clean_ppn.'*" class="fancyurl fancybox.iframe fi-info"> </a>';

    $sum_titles++;
    $sum_volumes += $row['Exemplare'];
    $sum_avg = $sum_avg + $row['5J-Umsatz/Ex'];

    $table .= '<tr>
                <td class="left">'.$clean_ppn.' '.$overlay_katalog.'</td>
                <td class="left">'.utf8_encode($row['Signatur']).'</td>
                <td class="left">'.$row['Indikator'].'</td>
                <td class="left">'.utf8_encode($row['Titel']).'</td>
                <td class="left">'.$clean_aut.'</td>
                <td>'.$clean_isbn.' '.$overlay_buchkatalog.'&nbsp;</td>
                <td>'.$row['Von'].'</td>
                <td>'.$row['Alter'].'</td>
                <td>'.$row['Exemplare'].'</td>
                <td '.tbl_td_class($row['Ausleihen 5 Jahre'])   .'>'.$row['Ausleihen 5 Jahre'].'</td>
                <td>'.$avg.'</td>
                <td '.tbl_td_class($row['Ausleihen lfd. Jahr']) .'>'.$row['Ausleihen lfd. Jahr'].'</td>
                <td '.tbl_td_class($row['Ausleihen J-1'])       .'>'.$row['Ausleihen J-1'].'</td>
                <td '.tbl_td_class($row['Ausleihen J-2'])       .'>'.$row['Ausleihen J-2'].'</td>
                <td '.tbl_td_class($row['Ausleihen J-3'])       .'>'.$row['Ausleihen J-3'].'</td>
                <td '.tbl_td_class($row['Ausleihen J-4'])       .'>'.$row['Ausleihen J-4'].'</td>
                <td '.tbl_td_class($row['Ausleihen J-5'])       .'>'.$row['Ausleihen J-5'].'</td>
                <td '.tbl_td_class($row['Vormerk. lfd. Jahr'])  .'>'.$row['Vormerk. lfd. Jahr'].'</td>
                <td '.tbl_td_class($row['Vormerk. J-1']).'>'     .$row['Vormerk. J-1'].'</td>
              </tr>';
  }
  $table .= '</tbody></table>';
}


/*##############################################################################
 * JavaScript: Lazy version to remember form settings
 #############################################################################*/
$js = "<script type=\"text/javascript\">
        setchecked('opt_location', '$signatur');

        function setchecked(selectid, set_val_selected) {
          optionen = document.getElementById(selectid).options;
          for(i = 0; i < optionen.length; i++) {
            if(optionen[i].value == set_val_selected) {
              optionen[i].setAttribute('selected','selected');
            }
          }
        }

        // JQUERY
        $(document).ready(function() {
          //$('#myTable').tablesorter({widgets: ['zebra']});

          var table = $('#myTable').DataTable({
              paging: false,
              'language': {
                'sEmptyTable':      'Keine Daten in der Tabelle vorhanden',
                'sInfo':            '_START_ bis _END_ von _TOTAL_ Einträgen',
                'sInfoEmpty':       '0 bis 0 von 0 Einträgen',
                'sInfoFiltered':    '(gefiltert von _MAX_ Einträgen)',
                'sInfoPostFix':     '',
                'sInfoThousands':   '.',
                'sLengthMenu':      '_MENU_ Einträge anzeigen',
                'sLoadingRecords':  'Wird geladen...',
                'sProcessing':      'Bitte warten...',
                'sSearch':          'Suchen',
                'sZeroRecords':     'Keine Einträge vorhanden.',
                'oPaginate': {
                  'sFirst':       'Erste',
                  'sPrevious':    'Zurück',
                  'sNext':        'Nächste',
                  'sLast':        'Letzte'
                },
                'oAria': {
                  'sSortAscending':  ': aktivieren, um Spalte aufsteigend zu sortieren',
                  'sSortDescending': ': aktivieren, um Spalte absteigend zu sortieren'
                }
              },
              'dom': 'C<\"clear\">lfrtip'
          } );

          new $.fn.dataTable.FixedHeader(table, {
            'offsetTop': 0
          } );

          //FANCYBOX
          $('.fancyurl').fancybox({
        		maxWidth	: 1600,
        		maxHeight	: 1200,
        		fitToView	: false,
        		width		: '95%',
        		height		: '95%',
        		autoSize	: false,
        		closeClick	: false,
        		openEffect	: 'none',
        		closeEffect	: 'none'
        	});

        } );
      </script>";
/*
,
                'dom': 'T<\"clear\">lfrtip',
                  'tableTools': {
                      'sSwfPath': '/../js/jquery_DataTables/extensions/TableTools/swf/copy_csv_xls_pdf.swf'
*/

$avg_total = ($sum_avg > 0) ? number_format ($sum_avg/$sum_titles, 2, ",", ".") : 0;
$output  = html_header($title, true);
$output .= '<div class="syb">';
$output .= "<h2>$title</h2>";
$output .= "<p>Stand: $now</p>";
$output .= '<p><strong>Titel gesamt</strong>: '.$sum_titles.'<br />
            <strong>Exemplare gesamt</strong>: '.$sum_volumes.'<br />
            <strong>5-Jahres-Durchschnittsumsatz/Exemplare</strong>: '.$avg_total.'</p>';
$output .= '<p><i>Anmerkung: 5 Jahre meint die letzen 5 vollen Jahre + dem Bruch des Jahres (2010, 2011, 2012, 2013, 2014 + 2015 bis jetzt).<br />
            Anmerkung2: <span class="fi-info"></span> = Link zum Katalog bzw. Buchkatalog.de</i></p>';
$output .= $form;
$output .= $table;
$output .= '</div>';
$output .= $overlay;
$output .= $js;
$output .= html_footer();

echo $output;
?>
